@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-USER; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-who/packages.lisp,v 1.4 2004/09/13 12:14:15 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-who/packages.lisp,v 1.9 2005/08/02 18:17:41 edi Exp $
 
-;;; Copyright (c) 2003-2004, Dr. Edmund Weitz. All rights reserved.
+;;; Copyright (c) 2003-2005, Dr. Edmund Weitz. All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -32,9 +32,12 @@
 #-:cormanlisp
 (defpackage #:cl-who
   (:use #:cl)
-  (:export #:*escape-char-p*
+  (:export #:*attribute-quote-char*
+           #:*escape-char-p*
            #:*prologue*
            #:conc
+           #:convert-attributes
+           #:convert-tag-to-string-list
            #:esc
            #:escape-string
            #:escape-string-all
@@ -43,6 +46,7 @@
            #:escape-string-minimal-plus-quotes
            #:fmt
            #:htm
+           #:html-mode
            #:show-html-expansion
            #:str
            #:with-html-output
@@ -51,7 +55,8 @@
 #+:cormanlisp
 (defpackage "CL-WHO"
   (:use "CL")
-  (:export "*ESCAPE-CHAR-P*"
+  (:export "*ATTRIBUTE-QUOTE-CHAR*" 
+           "*ESCAPE-CHAR-P*"
            "*PROLOGUE*"
            "CONC"
            "ESC"
@@ -62,6 +67,7 @@
            "ESCAPE-STRING-MINIMAL-PLUS-QUOTES"
            "FMT"
            "HTM"
+           "HTML-MODE"
            "SHOW-HTML-EXPANSION"
            "STR"
            "WITH-HTML-OUTPUT"
