@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package:CL-WHO; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-who/who.lisp,v 1.16 2004/04/24 00:05:39 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-who/who.lisp,v 1.22 2005/08/02 18:17:41 edi Exp $
 
-;;; Copyright (c) 2003-2004, Dr. Edmund Weitz. All rights reserved.
+;;; Copyright (c) 2003-2005, Dr. Edmund Weitz. All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -44,27 +44,57 @@ argument is T")
   "Whether to insert line breaks and indent. Also controls amount of
 indentation dynamically.")
 
+(defvar *html-mode* :xml
+  ":SGML for \(SGML-)HTML, :XML \(default) for XHTML.")
+
+(defparameter *attribute-quote-char* #\'
+  "Quote character for attributes.")
+
+(defparameter *empty-tag-end* " />"
+  "End of an empty tag. Default is XML style.")
+
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (unless (boundp '+newline+)
     (defconstant +newline+ (make-string 1 :initial-element #\Newline)
       "Used for indentation."))
 
   (unless (boundp '+spaces+)
-    (defconstant +spaces+ (make-string 2000 :initial-element #\Space)
+    (defconstant +spaces+ (make-string 2000
+                                       :initial-element #\Space
+                                       :element-type 'base-char)
       "Used for indentation.")))
 
 (defmacro n-spaces (n)
   "A string with N spaces - used by indentation."
   `(make-array ,n
-	      :element-type 'base-char
-	      :displaced-to +spaces+
-	      :displaced-index-offset 0))
+               :element-type 'base-char
+               :displaced-to +spaces+
+               :displaced-index-offset 0))
+
+(defun html-mode ()
+  "Returns the current HTML mode. :SGML for (SGML-)HTML and
+:XML for XHTML."
+  *html-mode*)
+
+(defun (setf html-mode) (mode)
+  "Sets the output mode to XHTML or \(SGML-)HTML.  MODE can be
+:SGML for HTML or :XML for XHTML."
+  (ecase mode
+    ((:sgml)
+     (setf *html-mode* :sgml
+           *empty-tag-end* ">"
+           *prologue* "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">"))
+    ((:xml)
+     (setf *html-mode* :xml
+           *empty-tag-end* " />"
+           *prologue* "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">"))))
 
 (defun escape-string (string &key (test *escape-char-p*))
   (declare (optimize speed))
   "Escape all characters in STRING which pass TEST. This function is
 not guaranteed to return a fresh string."
-  (let ((first-pos (position-if test string)))
+  (let ((first-pos (position-if test string))
+        (format-string (if (eq *html-mode* :xml) "&#x~x;" "&#~d;")))
     (if (not first-pos)
       ;; nothing to do, just return STRING
       string
@@ -90,7 +120,7 @@ not guaranteed to return a fresh string."
                    ((#\")
                      (write-sequence "&quot;" s))
                    (otherwise
-                     (format s "&#x~x;" (char-code char))))
+                     (format s format-string (char-code char))))
               while (< (1+ pos) len)
               finally (unless pos
                         (write-sequence string s :start old-pos)))))))
@@ -117,17 +147,20 @@ character set."
                                   (or (find char "<>&'\"")
                                       (> (char-code char) 127)))))
 
-(defun print-tag (list)
+(defun process-tag (sexp body-fn)
   (declare (optimize speed space))
-  "Returns a string list corresponding to the HTML tag TAG. Utility
-function used by TREE-TO-TEMPLATE."
+  "Returns a string list corresponding to the `HTML' \(in CL-WHO
+syntax) in SEXP.  Uses the generic function CONVERT-TO-STRING-LIST
+internally.  Utility function used by TREE-TO-TEMPLATE."
   (let (tag attr-list body)
     (cond
-      ((atom (first list))
-       (setq tag (first list))
+      ((keywordp sexp)
+       (setq tag sexp))
+      ((atom (first sexp))
+       (setq tag (first sexp))
        ;; collect attribute/value pairs into ATTR-LIST and tag body (if
        ;; any) into BODY
-       (loop for rest on (cdr list) by #'cddr
+       (loop for rest on (cdr sexp) by #'cddr
 	     if (keywordp (first rest))
 	       collect (cons (first rest) (second rest)) into attr
 	     else
@@ -135,14 +168,77 @@ function used by TREE-TO-TEMPLATE."
 			 (setq body rest)
 			 (return))
 	     finally (setq attr-list attr)))
-      ((listp (first list))
-       (setq tag (first (first list)))
-       (loop for rest on (cdr (first list)) by #'cddr
+      ((listp (first sexp))
+       (setq tag (first (first sexp)))
+       (loop for rest on (cdr (first sexp)) by #'cddr
 	     if (keywordp (first rest))
-	     collect (cons (first rest) (second rest)) into attr
+               collect (cons (first rest) (second rest)) into attr
 	     finally (setq attr-list attr))
-       (setq body (cdr list))))
+       (setq body (cdr sexp))))
+    (convert-tag-to-string-list tag attr-list body body-fn)))
 
+(defun convert-attributes (attr-list)
+  "Helper function for CONVERT-TAG-TO-STRING-LIST which converts the
+alist ATTR-LIST of attributes into a list of strings and/or Lisp
+forms."
+  (declare (optimize speed space))
+  (loop with =var= = (gensym)
+        with attribute-quote = (string *attribute-quote-char*)
+        for (attr . val) in attr-list
+        unless (null val) ;; no attribute at all if VAL is NIL
+          if (constantp val)
+            if (and (eq *html-mode* :sgml) (eq val t)) ; Special case for SGML
+              nconc (list " " (string-downcase attr))
+            else
+              nconc (list " "
+                          ;; name of attribute
+                          (string-downcase attr)
+                          (format nil "=~C" *attribute-quote-char*)
+                          ;; value of attribute
+                          (cond ((stringp val)
+                                 ;; a string, just use it - this case is
+                                 ;; actually not necessary because of
+                                 ;; the last case
+                                 val)
+                                ((eq val t)
+                                 ;; VAL is T, use attribute's name
+                                 (string-downcase attr))
+                                (t
+                                 ;; constant form, PRINC it
+                                 (format nil "~A" val)))
+                          attribute-quote)
+            end
+          else
+            ;; do the same things as above but at runtime
+            nconc (list `(let ((,=var= ,val))
+                          (cond ((null ,=var=))
+                                ((and (eq ,=var= t) (eq *html-mode* :xml))
+                                 (htm ,(format nil " ~A=~C~A~C"
+                                               (string-downcase attr)
+                                               *attribute-quote-char*
+                                               (string-downcase attr)
+                                               *attribute-quote-char*)))
+                                ((and (eq ,=var= t) (eq *html-mode* :sgml))
+                                 (htm ,(format nil " ~A"
+                                               (string-downcase attr))))
+                                (t
+                                 (htm ,(format nil " ~A=~C" (string-downcase attr)
+                                               *attribute-quote-char*)
+                                      (str ,=var=)
+                                      ,attribute-quote)))))))
+
+(defgeneric convert-tag-to-string-list (tag attr-list body body-fn)
+  (:documentation "Used by PROCESS-TAG to convert `HTML' into a list
+of strings.  TAG is a keyword symbol naming the outer tag, ATTR-LIST
+is an alist of its attributes \(the car is the attribute's name as a
+keyword, the cdr is its value), BODY is the tag's body, and BODY-FN is
+a function which should be applied to BODY.  The function must return
+a list of strings or Lisp forms."))
+
+(defmethod convert-tag-to-string-list (tag attr-list body body-fn)
+  "The standard method which is not specialized.  The idea is that you
+can use EQL specializers on the first argument."
+  (declare (optimize speed space))
     (nconc
      (if *indent*
        ;; indent by *INDENT* spaces
@@ -150,39 +246,8 @@ function used by TREE-TO-TEMPLATE."
      ;; tag name
      (list "<" (string-downcase tag))
      ;; attributes
-     (loop with =var= = (gensym)
-           for (attr . val) in attr-list
-           unless (null val) ;; no attribute at all if VAL is NIL
-           if (constantp val)
-           nconc (list " "
-                       ;; name of attribute
-                       (string-downcase attr)
-                       "='"
-                       ;; value of attribute
-                       (cond ((stringp val)
-                               ;; a string, just use it - this case is
-                               ;; actually not necessary because of
-                               ;; the last case
-                               val)
-                             ((eq val t)
-                               ;; VAL is T, use attribute's name
-                               (string-downcase attr))
-                             (t
-                               ;; constant form, PRINC it
-                               (format nil "~A" val)))
-                       "'")
-           else
-           ;; do the same things as above but at runtime
-           nconc (list `(let ((,=var= ,val))
-                         (cond ((null ,=var=))
-                               ((eq ,=var= t)
-                                 (htm ,(format nil " ~A='~A'"
-                                               (string-downcase attr)
-                                               (string-downcase attr))))
-                               (t
-                                 (htm ,(format nil " ~A='" (string-downcase attr))
-                                      (str ,=var=)
-                                      "'"))))))
+     (convert-attributes attr-list)
+     ;; body
      (if body
        (append
         (list ">")
@@ -190,15 +255,15 @@ function used by TREE-TO-TEMPLATE."
         ;; *INDENT* by 2 if necessary
         (if *indent*
           (let ((*indent* (+ 2 *indent*)))
-            (tree-to-template body))
-          (tree-to-template body))
+            (funcall body-fn body))
+          (funcall body-fn body))
         (if *indent*
           ;; indentation
           (list +newline+ (n-spaces *indent*)))
         ;; closing tag
         (list "</" (string-downcase tag) ">"))
        ;; no body, so no closing tag
-       (list " />")))))
+       (list *empty-tag-end*))))
 
 (defun apply-to-tree (function test tree)
   (declare (optimize speed space))
@@ -230,33 +295,23 @@ function used by TREE-TO-TEMPLATE and TREE-TO-COMMANDS-AUX."
   "Transforms an HTML tree into an intermediate format - mainly a
 flattened list of strings. Utility function used by TREE-TO-COMMANDS-AUX."
   (loop for element in tree
-        nconc (cond ((keywordp element)
-                      ;; stand-alone tag without content
-                      (list (concatenate 'string
-                                         (if *indent*
-                                           (concatenate 'string
-                                                        +newline+
-                                                        (n-spaces *indent*)))
-                                         "<"
-                                         (string-downcase element)
-                                         " />")))
-                    ((or
-		      (and (listp element)
-			   (keywordp (first element)))
-		      (and (listp element)
-			   (listp (first element))
-			   (keywordp (first (first element)))))
-                      ;; normal tag
-                      (print-tag element))
+        nconc (cond ((or (keywordp element)
+                         (and (listp element)
+                              (keywordp (first element)))
+                         (and (listp element)
+                              (listp (first element))
+                              (keywordp (first (first element)))))
+                     ;; normal tag
+                     (process-tag element #'tree-to-template))
                     ((listp element)
-                      ;; most likely a normal Lisp form - check
-                      ;; if we have nested HTM subtrees
-                      (list
-                       (replace-htm element #'tree-to-template)))
+                     ;; most likely a normal Lisp form - check if we
+                     ;; have nested HTM subtrees
+                     (list
+                      (replace-htm element #'tree-to-template)))
                     (t
-                      (if *indent*
-                        (list +newline+ (n-spaces *indent*) element)
-                        (list element))))))
+                     (if *indent*
+                       (list +newline+ (n-spaces *indent*) element)
+                       (list element))))))
 
 (defun string-list-to-string (string-list)
   (declare (optimize speed space))
@@ -330,7 +385,7 @@ for REPLACE-HTM."
                                               collector))
                               (nreverse collector)))))))
 
-(defun tree-to-commands (tree stream prologue)
+(defun tree-to-commands (tree stream &optional prologue)
   (declare (optimize speed space))
   "Transforms an HTML tree into code to print the HTML to STREAM."
   ;; use TREE-TO-TEMPLATE, then TREE-TO-COMMANDS-AUX, and finally
@@ -426,4 +481,3 @@ into Lisp code which creates the corresponding HTML as a string."
     (cdr (assoc symbol
                 exported-symbols-alist
                 :test #'eq))))
-               
\ No newline at end of file
